@@ -1,6 +1,6 @@
 # Distributions menu dialogs for selecting samples
 
-# last modified 2014-10-02 by J. Fox
+# last modified 2016-07-14 by J. Fox
 # modified by Miroslav M. Ristic (15 January 2011)
 
 
@@ -27,7 +27,7 @@ distributionSamples <- function(nameVar) {
   fVar<-get(paste(nameVar,"Distribution",sep=""))
   nnVar<-length(fVar$params)
   dialogName <- paste(nameVar,"DistributionSamples", sep="")
-  defaults <- list(initialValues=fVar$initialValues, dsname=gettextRcmdr(paste(fVar$titleName,"Samples",sep="")),
+  defaults <- list(initialValues=fVar$initialValues, dsname=paste(fVar$titleName,"Samples",sep=""),
                    nobs="100", nsamples="1", mean="1", sum="0", sd="0")
   initial <- getDialog(dialogName, defaults=defaults)
   initializeDialog(title=gettextRcmdr(paste("Sample from ",fVar$titleName," Distribution")))
@@ -103,11 +103,12 @@ distributionSamples <- function(nameVar) {
     for (i in 1:nnVar) {
       pasteVar<-paste(pasteVar,", ",fVar$params[i],"=",vars[i],sep="")
     }
-    command.1 <- if (nameVar=="Gumbel") {
-      paste(dsnameValue, " <- as.data.frame(matrix(log(rweibull(", samples, "*", obser, ", shape=", vars[1], ", scale=", vars[2], ")), ncol=", obser, "))", sep="")
-    } else {
+    command.1 <- 
+    #     if (nameVar=="Gumbel") {
+    #   paste(dsnameValue, " <- as.data.frame(matrix(log(rweibull(", samples, "*", obser, ", shape=", vars[1], ", scale=", vars[2], ")), ncol=", obser, "))", sep="")
+    # } else {
       paste(dsnameValue, " <- as.data.frame(matrix(r",fVar$funName,"(", samples, "*", obser, pasteVar, "), ncol=", obser, "))", sep="")   
-    }
+    # }
     doItAndPrint(command.1)
     command.1 <- if (samples == 1) 
       paste("rownames(", dsnameValue, ') <- "sample"', sep="")
